#include "serial.h"
#include "mqtt.h"
#include "delay.h"
#include "gps.h"

int fd1,fd2;

int init_serial(const char *port, int baudrate) {
    int fd = open(port, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1) {
        perror("open serial port");
        return -1;
    }

    struct termios options;
    tcgetattr(fd, &options);

    // 设置波特率
    cfsetispeed(&options, B9600);
    cfsetospeed(&options, B9600);

    // 配置串口
    options.c_cflag &= ~PARENB; // 无校验
    options.c_cflag &= ~CSTOPB; // 1停止位
    options.c_cflag &= ~CSIZE;  // 清除数据位设置
    options.c_cflag |= CS8;     // 8数据位
    options.c_cflag |= CLOCAL | CREAD; // 忽略调制解调器状态，开启接收器

    options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // 原始模式
    options.c_iflag &= ~(IXON | IXOFF | IXANY);         // 禁用流控
    options.c_oflag &= ~OPOST;                         // 原始输出

    tcsetattr(fd, TCSANOW, &options);
    return fd;
}


void serial_Receive_2()
{
    char buffer[BUFFER_SIZE];
    char nmea_sentence[BUFFER_SIZE];
    int index=0;
    //mosq = mosquitto_new(CLIENT_ID, true, NULL);
    while (1) 
    { 
        int bytes_read = read(fd2, buffer, BUFFER_SIZE-1);
        if (bytes_read > 0) 
        {
            buffer[bytes_read] = '\0';
            for (int i = 0; i < bytes_read; i++) 
            {
                if (buffer[i] == '$') 
                {
                    index = 0; // 新语句开始
                }
                if (index < BUFFER_SIZE - 1) 
                {
                    nmea_sentence[index++] = buffer[i];
                    nmea_sentence[index] = '\0';
                }
                if (buffer[i] == '\n') 
                { // 一条语句结束
                    if (strstr(nmea_sentence, "$GNRMC")) 
                    {
                        parse_gnrmc(nmea_sentence); // 解析$GNRMC语句
                        
                    }
                }
            }   
        }
     Delay_Ms(2,0);
    }
    close(fd2);
}

void serial_Receive_1() 
{
    unsigned char buffer[254];

    while(1) 
    { 
        //printf("sada\n");
        int bytes_read =read(fd1, buffer, sizeof(buffer)-1);
        
        if (bytes_read > 0) {
             //printf("Received chatdate: %s\n", buffer);
            buffer[bytes_read] = 0xfe;
            printf("read bite: %d\n", bytes_read);
            printf("Received chatdate: %d\n", buffer[0]); // 以十六进制打印每个字节
            printf("Received chatdate: %d\n", buffer[1]);
            printf("Received chatdate: %d\n", buffer[2]);
            printf("Received chatdate: %d\n", buffer[3]);
            //int receivedNumber = atoi(buffer);
            //printf("Received intdate: %d\n", receivedNumber);
        }
         
        Delay_Ms(2,0); 
    }
    close(fd1);
}

void serial_Write(unsigned char *buf)
{
    unsigned char buffers[5];
    if(gmessage_type==3)
    {
        buffers[0]=0xAA;
        //write(fd, buffers[0], sizeof(buffers));
        for(int i=1;i<4;i++)
        {
            buffers[i]=buf[i-1];
            //write(fd, buffers[i], sizeof(buffers));
        }
        buffers[4]=0xFE;
        write(fd1, buffers, sizeof(buffers));
    }
    if(gmessage_type==4)
    {
        buffers[0]=0xAA;
        buffers[1]=0xEE;
        //write(fd, buffers[0], sizeof(buffers));
        for(int i=2;i<4;i++)
        {
            buffers[i]=buf[i-1];
            //write(fd, buffers[i], sizeof(buffers));
        }
        buffers[4]=0xFE;
        write(fd1, buffers, sizeof(buffers));
    }
    

}



int serial_Init()
{
    fd1 = init_serial(SERIAL_PORT1, 9600);
    fd2 = init_serial(SERIAL_PORT2, 9600);
    if (fd1 == -1) {
        return -1;
    printf("Serial 1 communication started...\n");
    }
    if (fd2 == -1) {
        return -1;
    printf("Serial 2 communication started...\n");
    }
    
}
